from enum import Enum

BASEURL = "https://apis.issou.best/ordr"
WSURL = "https://ordr-ws.issou.best"

WSERRCODE = {
    1: "Unknown error triggered by an emergency stop.",
    2: "Replay download error (bad upload from the sender)",
    3: "Replay download error (bad download from the server), can happen because of invalid characters",
    4: "All beatmap mirrors are unavailable or the beatmap isn't yet available on them, try again later.",
    15: "Beatmap not found on the mirrors. Retry later.",
    18: "Unknown error from the renderer. Try to send the replay again.",
    19: "The renderer cannot download the map. This may be a server-side problem from issou.best.",
    20: "The beatmap version on the mirror is not the same as the replay. Don't retry sending it as we have already retried for you.",
    21: "This replay is corrupted. Try re-exporting it.",
    22: "There is a problem with the generated video. Try to send the replay again.",
    27: "Something went wrong, the generated video is not your replay. Try to send the replay again.",
    28: "The renderer cannot download the replay. Try to send the replay again.",
}
APIERRCODE = {
    2: "Cannot save replay.",
    5: "Cannot parse replay.",
    6: "This replay is not an osu!standard replay, o!rdr does not support mania/ctb/taiko.",
    7: "This replay has no input data. Try re-exporting it from osu!.",
    8: "This beatmap does not exist on osu!. Custom difficulties or non-submitted maps are not supported.",
    9: "The audio for this map is unavailable, it maybe has been copyright claimed.",
    10: "Cannot connect to osu! api. Please retry later.",
    11: "Auto mod is not supported.",
    12: "This replay has an invalid username.",
    13: "Beatmaps longer than 15 minutes are not allowed.",
    14: "This player is banned from o!rdr.",
    16: "This IP is banned from o!rdr.",
    17: "This username is banned from o!rdr.",
    23: "Failed to prepare the render. Try to send the replay again.",
    24: "This map has no name.",
    25: "This replay is missing input data.",
    26: "This replay has incompatible mods.",
    29: "This replay is already rendering or in queue. Wait until it finishes.",
    30: "Beatmaps with a star rating greater than 20 are not allowed.",
    31: "This mapper is blacklisted.",
    32: "This beatmapset is blacklisted.",
    33: "This replay has already errored less than an hour ago, try again later.",
}

class DeveloperMode(Enum):
    SUCCESS = 0
    FAIL = 1
    WSFAIL = 2

class Resolution(Enum):
    X480 = "720x480"
    X540 = "960x540"
    X720 = "1280x720"


import json

from ordr.classes import Render, Server, Skin

class GetSkinRespond:
    def __init__(self, status, data) -> None:
        self.statusCode = status
        self.rawcontent = data
        try:
            self._json = json.loads(data.decode("utf-8"))
        except json.JSONDecodeError:
            self.message = data.decode("utf-8")
        else:
            self.message = self._json["message"] if "message" in self._json else None
            self.maxSkins = self._json["maxSkins"] if "maxSkins" in self._json else None
            if "skins" in self._json:
                self.skins = []
                for i in self._json["skins"]:
                    self.skins.append(Skin.get_skin(i))

class CustomSkinRespond:
    def __init__(self, status, data) -> None:
        self.statusCode = status
        self.rawcontent = data
        try:
            self._json = json.loads(data.decode("utf-8"))
        except json.JSONDecodeError:
            self.message = data.decode("utf-8")
        else:
            self.found = self._json["found"]
            self.removed = self._json["removed"]
            self.message = self._json["message"]
            if self.found == True:
                self.skinName = self._json["skinName"]
                self.skinAuthor = self._json["skinAuthor"]
                self.downloadLink = self._json["downloadLink"]


class GetRenderRespond:
    def __init__(self, status, data) -> None:
        self.statusCode = status
        self.rawcontent = data
        try:
            self._json = json.loads(data.decode("utf-8"))
        except json.JSONDecodeError:
            self.message = data.decode("utf-8")
        else:
            self.maxRenders = self._json["maxRenders"] if "maxRenders" in self._json else None
            if "renders" in self._json:
                self.renders = []
                for i in self._json["server"]:
                    self.renders.append(Render(i))

class NewRenderRespond:
    def __init__(self, status, data) -> None:
        self.statusCode = status
        self.rawcontent = data
        try:
            self._json = json.loads(data.decode("utf-8"))
        except json.JSONDecodeError:
            self.message = data.decode("utf-8")
        else:
            self.message = self._json["message"] if "message" in self._json else None
            self.renderID = self._json["renderID"] if "renderID" in self._json else None
            self.error = self._json["errorCode"] if "errorCode" in self._json else None
            self.reason = self._json["reason"] if "reason" in self._json else None

class GetServersRespond:
    def __init__(self, status, data) -> None:
        self.statusCode = status
        self.rawcontent = data
        try:
            self._json = json.loads(data.decode("utf-8"))
        except json.JSONDecodeError:
            self.message = data.decode("utf-8")
        else:
            if "servers" in self._json:
                self.servers = []
                for i in self._json["server"]:
                    self.servers.append(Server(i))


class Server:
    def __init__(self, json) -> None:
        self._json = json
        self.enabled = json["enabled"]
        self.lastSeen = json["lastSeen"]
        self.name = json["name"]
        self.priority = json["priority"]
        self.oldScore = json["oldScore"]
        self.avgFPS = json["avgFPS"]
        self.power = json["power"]
        self.status = json["status"]
        self.totalRendered = json["totalRendered"]
        self.renderingType = json["renderingType"]
        self.cpu = json["cpu"]
        self.gpu = json["gpu"]
        self.motionBlurCapable = json["motionBlurCapable"]
        self.usingOsuApi = json["usingOsuApi"]
        self.uhdCapable = json["uhdCapable"]
        self.avgRenderTime = json["avgRenderTime"]
        self.avgUploadTime = json["avgUploadTime"]
        self.totalAvgTime = json["totalAvgTime"]
        self.totalUploadedVideosSize = json["totalUploadedVideosSize"]

class Skin:
    def __init__(self, skinID:int, isCustomSkin:bool) -> None:
        self.id = skinID
        self.customSkin = isCustomSkin
    
    def get_skin(self, json):
        self._json = json
        self.skin = json["skin"]
        self.presentationName = json["presentationName"]
        self.url = json["url"]
        self.highResPreview = json["highResPreview"]
        self.lowResPreview = json["lowResPreview"]
        self.gridPreview = json["gridPreview"]
        self.id = json["id"]
        self.hasCursorMiddle = json["hasCursorMiddle"]
        self.author = json["author"]
        self.modified = json["modified"]
        self.version = json["version"]
        self.alphabeticalId = json["alphabeticalId"]
        self.timesUsed = json["timesUsed"]
        return self

class Render:
    def __init__(self, json) -> None:
        self._json = json
        self.date = json["date"]
        self.readableDate = json["readableDate"]
        self.renderID = json["renderID"]
        self.username = json["username"]
        self.progress = json["progress"]
        self.errorCode = json["errorCode"]
        self.removed = json["removed"]
        self.renderer = json["renderer"]
        self.description = json["description"]
        self.title = json["title"]
        self.replayFilePath = json["replayFilePath"]
        self.replayMd5 = json["replayMd5"]
        self.videoUrl = json["videoUrl"]
        self.mapLink = json["mapLink"]
        self.mapTitle = json["mapTitle"]
        self.mapLength = json["mapLength"]
        self.drainTime = json["drainTime"]
        self.replayDifficulty = json["replayDifficulty"]
        self.replayUsername = json["replayUsername"]
        self.replayMods = json["replayMods"]
        self.mapID = json["mapID"]
        self.needToRedownload = json["needToRedownload"]
        self.resolution = json["resolution"]
        self.globalVolume = json["globalVolume"]
        self.musicVolume = json["musicVolume"]
        self.hitsoundVolume = json["hitsoundVolume"]
        self.useSkinHitsounds = json["useSkinHitsounds"]
        self.playNightcoreSamples = json["playNightcoreSamples"]
        self.showHitErrorMeter = json["showHitErrorMeter"]
        self.showUnstableRate = json["showUnstableRate"]
        self.showScore = json["showScore"]
        self.showHPBar = json["showHPBar"]
        self.showComboCounter = json["showComboCounter"]
        self.showPPCounter = json["showPPCounter"]
        self.showKeyOverlay = json["showKeyOverlay"]
        self.showScoreboard = json["showScoreboard"]
        self.showAvatarsOnScoreboard = json["showAvatarsOnScoreboard"]
        self.showBorders = json["showBorders"]
        self.showMods = json["showMods"]
        self.showResultScreen = json["showResultScreen"]
        self.showHitCounter = json["showHitCounter"]
        self.showAimErrorMeter = json["showAimErrorMeter"]
        self.customSkin = json["customSkin"]
        self.skin = json["skin"]
        self.hasCursorMiddle = json["hasCursorMiddle"]
        self.useSkinCursor = json["useSkinCursor"]
        self.useSkinColors = json["useSkinColors"]
        self.useBeatmapColors = json["useBeatmapColors"]
        self.cursorScaleToCS = json["cursorScaleToCS"]
        self.cursorRainbow = json["cursorRainbow"]
        self.cursorTrailGlow = json["cursorTrailGlow"]
        self.cursorSize = json["cursorSize"]
        self.cursorTrail = json["cursorTrail"]
        self.drawFollowPoints = json["drawFollowPoints"]
        self.drawComboNumbers = json["drawComboNumbers"]
        self.scaleToTheBeat = json["scaleToTheBeat"]
        self.sliderMerge = json["sliderMerge"]
        self.objectsRainbow = json["objectsRainbow"]
        self.objectsFlashToTheBeat = json["objectsFlashToTheBeat"]
        self.useHitCircleColor = json["useHitCircleColor"]
        self.seizureWarning = json["seizureWarning"]
        self.loadStoryboard = json["loadStoryboard"]
        self.loadVideo = json["loadVideo"]
        self.introBGDim = json["introBGDim"]
        self.inGameBGDim = json["inGameBGDim"]
        self.breakBGDim = json["breakBGDim"]
        self.BGParallax = json["BGParallax"]
        self.showDanserLogo = json["showDanserLogo"]
        self.motionBlur960fps = json["motionBlur960fps"]
        self.motionBlurForce = json["motionBlurForce"]
        self.skip = json["skip"]
        self.cursorRipples = json["cursorRipples"]
        self.sliderSnakingIn = json["sliderSnakingIn"]
        self.sliderSnakingOut = json["sliderSnakingOut"]
        self.isVerified = json["isVerified"]
        self.isBot = json["isBot"]
        self.renderStartTime = json["renderStartTime"]
        self.renderEndTime = json["renderEndTime"]
        self.renderTotalTime = json["renderTotalTime"]
        self.uploadEndTime = json["uploadEndTime"]
        self.uploadTotalTime = json["uploadTotalTime"]


from setuptools import setup, find_packages

VERSION = '0.0.2' 
DESCRIPTION = 'A simple o!rdr API and Websocket wrapper.'
LONG_DESCRIPTION = 'ordr.py is a simple and easy to use o!rdr API and Websocket wrapper.'

setup(
    name="ordr.py", 
    version=VERSION,
    author="Ryan",
    author_email="itsrql@gmail.com",
    description=DESCRIPTION,
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/ItsRqtl/ordr.py",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.10"
    ],
    packages=find_packages(),
    install_requires=[
        "requests", 
        "python-socketio",
        "aiohttp"
    ],
)


from socketio import Client
from ordr.classes import Skin, Server
from ordr.respond import *
from ordr.constant import *
import requests
from requests.structures import CaseInsensitiveDict

class OrdrClient:

    socket = Client()

    def __init__(self, apikey:str=None, developer:DeveloperMode=None) -> None:
        if developer == DeveloperMode.SUCCESS: apikey = "devmode_success"
        elif developer == DeveloperMode.FAIL: apikey = "devmode_fail"
        elif developer == DeveloperMode.WSFAIL: apikey = "devmode_wsfail"
        self.apikey = apikey
    
    def connect(self):
        self.socket.connect(url=WSURL)

    def disconnect(self):
        self.socket.disconnect()

    def create_render(self, replayURL:str, username:str, skin:Skin, resolution:Resolution=Resolution.X720, globalVolume:int=50, musicVolume:int=50, hitsoundVolume:int=50, showHitErrorMeter:bool=True, showUnstableRate:bool=True, showScore:bool=True, showHPBar:bool=True, showComboCounter:bool=True, showPPCounter:bool=True, showScoreboard:bool=False, showBorders:bool=False, showMods:bool=True, showResultScreen:bool=True, useSkinCursor:bool=True, useSkinColors:bool=False, useSkinHitsounds:bool=True, useBeatmapColors:bool=True, cursorRainbow:bool=False, cursorTrailGlow:bool=False, drawFollowPoints:bool=True, scaleToTheBeat:bool=False, sliderMerge:bool=False, objectsRainbow:bool=False, objectsFlashToTheBeat:bool=False, useHitCircleColor:bool=True, seizureWarning:bool=False, loadStoryboard:bool=True, loadVideo:bool=True, introBGDim:int=0, inGameBGDim:int=75, breakBGDim:int=30, BGParallax:bool=False, showDanserLogo:bool=True, skip:bool=True, cursorRipples:bool=False, cursorSize:float=1, cursorTrail:bool=True, drawComboNumbers:bool=True, sliderSnakingIn:bool=True, sliderSnakingOut:bool=True, showHitCounter:bool=False, showKeyOverlay:bool=True, showAvatarsOnScoreboard:bool=False, showAimErrorMeter:bool=False, playNightcoreSamples:bool=True):
        headers = CaseInsensitiveDict()
        headers["Content-Type"] = "application/x-www-form-urlencoded"
        data = f"replayURL={replayURL}&username={username}&resolution={resolution}&skin={skin.id}&customSkin={skin.customSkin}&globalVolume={globalVolume}&musicVolume={musicVolume}&hitsoundVolume={hitsoundVolume}&showHitErrorMeter={showHitErrorMeter}&showUnstableRate={showUnstableRate}&showScore={showScore}&showHPBar={showHPBar}&showComboCounter={showComboCounter}&showPPCounter={showPPCounter}&showScoreboard={showScoreboard}&showBorders={showBorders}&showMods={showMods}&showResultScreen={showResultScreen}&useSkinCursor={useSkinCursor}&useSkinColors={useSkinColors}&useSkinHitsounds={useSkinHitsounds}&useBeatmapColors={useBeatmapColors}&cursorRainbow={cursorRainbow}&cursorTrailGlow={cursorTrailGlow}&drawFollowPoints={drawFollowPoints}&scaleToTheBeat={scaleToTheBeat}&sliderMerge={sliderMerge}&objectsRainbow={objectsRainbow}&objectsFlashToTheBeat={objectsFlashToTheBeat}&useHitCircleColor={useHitCircleColor}&seizureWarning={seizureWarning}&loadStoryboard={loadStoryboard}&loadVideo={loadVideo}&introBGDim={introBGDim}&inGameBGDim={inGameBGDim}&breakBGDim={breakBGDim}&BGParallax={BGParallax}&showDanserLogo={showDanserLogo}&skip={skip}&cursorRipples={cursorRipples}&cursorSize={cursorSize}&cursorTrail={cursorTrail}&drawComboNumbers={drawComboNumbers}&sliderSnakingIn={sliderSnakingIn}&sliderSnakingOut={sliderSnakingOut}&showHitCounter={showHitCounter}&showKeyOverlay={showKeyOverlay}&showAvatarsOnScoreboard={showAvatarsOnScoreboard}&showAimErrorMeter={showAimErrorMeter}&playNightcoreSamples={playNightcoreSamples}&verificationKey={self.apikey}".replace("True", "true").replace("False", "false")
        resp = requests.post(f"{BASEURL}/renders", headers=headers, data=data)
        return NewRenderRespond(resp.status_code, resp.content)

    def get_render(self, pageSize:int=50, page:int=1, ordrUsername:str=None, replayUsername:str=None, renderID:int=None, nobots:bool=False, link:str=None, beatmapsetid:int=None):
        resp = requests.get(f"{BASEURL}/renders", params={"pageSize": pageSize, "page": page, "ordrUsername": ordrUsername, "replayUsername": replayUsername, "renderID": renderID, "nobots": f"{nobots}".lower(), "link": link, "beatmapsetid": beatmapsetid})
        return GetRenderRespond(resp.status_code, resp.content)

    def get_skin(self, pageSize:int=100, page:int=1, search:str=None):
        resp = requests.get(f"{BASEURL}/skins", params={"pageSize": pageSize, "page": page, "search": search})
        return GetSkinRespond(resp.status_code, resp.content)

    def get_custom_skin(self, id:int):
        resp = requests.get(f"{BASEURL}/skins/custom", params={"id": id})
        return CustomSkinRespond(resp.status_code, resp.content)

    def get_online_count(self):
        resp = requests.get(f"{BASEURL}/servers/onlinecount")
        return int(resp.content.decode("utf-8")) if resp.status_code == 200 else None

    def get_servers(self):
        resp = requests.get(f"{BASEURL}/servers")
        return GetServersRespond(resp.status_code, resp.content)